/********************************************************
 * GAMEMENU						*
 *							*
 * gamemenu.c						*
 * Version 1.0						*
 *							*
 * Changelog						*
 *							*
 * 1.0	-	2018/05/28	S.R.Usher		*
 *		Initial version.			*
 *							*
 ********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <limits.h>
#include <sys/utsname.h>

#include "gamemenu.h"

int has_a_speaker = 0;
int has_dma_audio = 0;
int has_a_68030 = 0;
int is_a_tt = 0;

void main(int argc, char **argv, char **envp)
{
	FILE *csv_file;
	char *basedir;
	char *game_desc[MAXGAMES], *game_dir[MAXGAMES], *game_prog[MAXGAMES];
	char scratch[PATH_MAX];
	int i, numgames, which_game;
	char csv_path[PATH_MAX];
	struct ini_options options;
	struct utsname uname_info;

/*
 * First we need to work out what type of machine we're running on
 * using the mintlibs' UNIX-compatible uname() function.

	uname(&uname_info);

#ifdef DEBUG
	printf("System name:\t%s\nNode name:\t%s\nRelease:\t%s\nVersion:\t%s\nMachine:\t%s\n",
	       uname_info.sysname, uname_info.nodename, uname_info.release,
	       uname_info.version, uname_info.machine);

	sleep(5);
#endif

/*
 * If we're running on a TT then that has DMA audio, a speaker and
 * a 68030 processor.
 */

	if (!strcmp(uname_info.machine, "ataritt"))
	{
		has_dma_audio = 1;
		has_a_speaker = 1;
		has_a_68030 = 1;
		is_a_tt = 1;
	}

/*
 * If we are on an STe then we just have DMA audio.
 */

	if (!strcmp(uname_info.machine, "atariste"))
	{
		has_dma_audio = 1;
	}

/*
 * Change the screen resolution and colours.
 */

	set_screen_mode();

/*
 * Read and parse the INI file. When run from the AUTO folder our current directory
 * is actually the root of the drive.
 */

	if ((basedir = read_ini_file(&options)) == NULL)
	{
		fprintf(stderr, "%s: error reading INI file: %s/%s\n", argv[0], getcwd(scratch, PATH_MAX), INI_FILE);
		sleep(5);
		exit(-1);
	}

#ifdef DEBUG
	printf("Game directory is '%s'\n", basedir);
#endif

/*
 * Change our current working directory to where the games are so we can use
 * relative path names from here on.
 */

	if (chdir(basedir) < 0)
	{
		perror("Can't change to game directory");
		exit(-1);
	}

/*
 * Open the CSV file for reading.
 */

	if ((csv_file = fopen(CSV_FILE, "r")) == NULL)
	{
		perror("Opening games list csv file\n");
		exit(-1);
	}

/*
 * Parse the file and put the data in the storage arrays, returning the number of entries.
 */

	numgames = parse_csv_file(csv_file, MAXGAMES, game_desc, game_dir, game_prog);

	fclose(csv_file);

/*
 * If there's been an error reading the CSV file then abort.
 */

	if (numgames < 1)
	{
		fprintf(stderr, "Error: No games to run.\n");
		exit(-1);
	}

/*
 * Now run the menu system giving it the list of game descriptions and the number of games.
 *
 * If it's negative then we've been told to quit. Restore the screen resolution and colours
 * back to how they were when we started and exit.
 */

	if ((which_game = run_menu(numgames, game_desc)) < 0)
	{
		unset_screen_mode();
		exit(0);
	}

/*
 * Check if by some fluke of nature the menu system has returned a value greater than the
 * length of the list.
 */

	if (which_game > (numgames - 1))
	{
		fprintf(stderr, "Abort! Selecting a non-existant game: Number of games %d, game selected %d\n", numgames, which_game + 1);
		unset_screen_mode();
		exit(-1);
	}

/*
 * If the machine has DMA audio and the option file has asked for it to be set then
 * call the routine to set the volume.
 */
	if ((options.master_volume >= 0 ) && has_dma_audio)
		set_volume(options.master_volume);

/*
 * If the machine has speaker and the option file has asked for it to be changed then
 * call the routine to turn it on or off.
 */
	if ((options.internal_speaker >= 0 ) && has_a_speaker)
		set_speaker(options.internal_speaker);

/*
 * Attempt to run the game.
 */
 
	run_game(game_dir[which_game], game_prog[which_game], argc, argv, envp);

/*
 * If we've got here then the game hasn't run. Clean up the screen and exit.
 */

	unset_screen_mode();
}
