/********************************************************
 * GAMEMENU						*
 *							*
 * menu.c						*
 * Version 1.0						*
 *							*
 * Changelog						*
 *							*
 * 1.0	-	2018/05/28	S.R.Usher		*
 *		Initial version.			*
 *							*
 ********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <curses.h>

#include "gamemenu.h"

#define HEADER_SIZE 5

int max_x, max_y;
int menu_max_x, menu_max_y;

int deploy_menu();

int run_menu(int total_games, char *game_desc[])
{
	int i, j, retval;
	int ypos, xstart;
	int half_width, menu_height;

	WINDOW *main, *menu;

	char *header_title = "--- Game menu ---";
	char *header_instrs = "N - Next, P - Previous, ENTER - Select, Q - Quit";

	if ((main = initscr()) == NULL)
	{
		perror("Can't initialise curses main window");
		exit(-1);
	}
	
	cbreak();
	noecho();
	nonl();

#ifdef NOTBROKEN
	getmaxyx(stdscr, max_y, max_x);
#else
/* Guess the size of the screen in characters */

	max_x = 80;
	max_y = 24;
#endif

	clear();
	refresh();

/*
 * Determine the centre of the screen and set the menu size.
 */
 
	half_width = max_x / 2;
	menu_height = max_y - HEADER_SIZE;

	menu_max_x = max_x;
	menu_max_y = menu_height;

	if ((menu = newwin(menu_max_y, menu_max_x, HEADER_SIZE, 0)) == NULL)
	{
		endwin();
		perror("Can't initialise curses menu window");
		exit(-1);
	}

	scrollok(menu, 1);

/*
 * Print the header, making the top header "bold"
 */
 
	standout();

	move(0, (max_x - strlen(header_title)) / 2);
	printw(header_title);

	standend();

	move(2, (max_x - strlen(header_instrs)) / 2);
	printw(header_instrs);

/*
 * Make sure that the header's updated and sent to the display before starting on the menu.
 */

	refresh();

	retval = deploy_menu(menu, menu_max_x, menu_max_y, total_games, game_desc);

	endwin();

	return retval;
}

int deploy_menu(WINDOW *win, int win_width, int win_height, int num_games, char *game_desc[])
{
	int i;
	int cur_game, top_menu_game, bot_menu_game, prev_game;
	int chosen = 0;
	int key;
	int desc_start[MAXGAMES];
	char blank_line[512];

/*
 * Work out the starting location of each description string so that they will
 * look centred and store them in an array.
 */

	prev_game = cur_game = 0;
	top_menu_game = 0;
	bot_menu_game = win_height - 1;

	for (i = 0; i < num_games; i++)
	{
		desc_start[i] = (win_width - strlen(game_desc[i])) / 2;
	}

/*
 * Create a prototype blank line.
 */

	for (i = 0; i < win_width; i++)
		blank_line[i] = ' ';

	blank_line[win_width] = '\0';

/*
 * Print out the initial background of descriptions which will be modified later
 * as we change the highlighted entry and scroll up and down the list.
 */

	for (i = 0; i < bot_menu_game; i++)
	{
		wmove(win, i, desc_start[i]);
		wprintw(win, game_desc[i]);
	}

	wrefresh(win);

/*
 * Now the main loop.
 */
	
	while(chosen == 0)
	{
/*
 * If the previously highlighted entry is within those displayed then blank the
 * line and re-print the entry unhighlighted.
 */
		if ((prev_game >= top_menu_game) && (prev_game < bot_menu_game))
		{
			wmove(win, prev_game - top_menu_game, 0);
			wprintw(win, "%s", blank_line);
			wmove(win, prev_game - top_menu_game, desc_start[prev_game]);
			wprintw(win, game_desc[prev_game]);
			wrefresh(win);
		}
/*
 * If the current game is above those displayed then we need to scroll down by one
 */
		if (cur_game < top_menu_game)
		{
			top_menu_game--;
			bot_menu_game--;
		/* Redraw everything as BSD curses can't scroll backwards. */

			wclear(win);
			
			for (i = top_menu_game; i < bot_menu_game;  i++)
			{
				wmove(win, i - top_menu_game, desc_start[i]);
				wprintw(win, game_desc[i]);
			}
		}
/*
 * If the current game is beyond the bottom of the displayed list then scroll up by one.
 */
		if (cur_game >= bot_menu_game)
		{
			scroll(win);
			top_menu_game++;
			bot_menu_game++;
		}
/*
 * Now print the currently selected entry in bold.
 */
 
		wstandout(win);
		wmove(win, cur_game - top_menu_game, 0);
		wprintw(win, "%s", blank_line);
		wmove(win, cur_game - top_menu_game, desc_start[cur_game]);
		wprintw(win, game_desc[cur_game]);
		wstandend(win);

		wrefresh(win);
/*
 * Read the keyboard and act upon it.
 */
		key = wgetch(win);

		switch (key)
		{
			case 'n':
			case 'B':
				if (cur_game < (num_games -1))
				{
					prev_game = cur_game;
					cur_game++;
				}
				break;
			case 'p':
			case 'A':
				if (cur_game > 0)
				{
					prev_game = cur_game;
					cur_game--;
				}
				break;
/*
 * To quit the loop we have to give a 'chosen' a non-zero value.
 *
 * To quit the program we need to give a non-positive value
 * for the chosen game so that the main program knows what to do.
 */
			case 'q':
				chosen = cur_game = -1;
				break;
			case 0x0d:
			case ' ':
				chosen = 1;
				break;
		}
	}

	return cur_game;
}
